@@ -208,7 +208,7 @@ class AuthenticationAborted (Exception):
         Exception.__init__ (self)
 
 
-class GnomeURLopener (urllib.FancyURLopener, gobject.GObject):
+class GnomeURLopener (urllib.FancyURLopener):
     """
     A subclass of C{urllib.URLopener} able to intercept user/password requests
     and pass them through an L{Account}, displaying a L{AccountDialog} if
@@ -219,18 +219,24 @@ class GnomeURLopener (urllib.FancyURLopener, gobject.GObject):
                      a new fresh GnomeURLopener each time you need one
     """
     
-    __gsignals__ = {
-        "done" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [gobject.TYPE_PYOBJECT]),
-    }
-    
-    def __init__ (self, account):
+    def __init__ (self, account, extra_widget_factory=None):
+        """
+        @param account: The L{Account} object to request credentials from
+        @param extra_widget_factory: This optional parameter may point to a
+                                     C{callable} returning a C{gtk.Widget} which
+                                     will be packed into the L{AccountDialog}
+                                     when it is spawned. The callable may also
+                                     return C{None} if no widgets should be
+                                     added.
+        """
         proxies = get_proxy()
         urllib.FancyURLopener.__init__ (self, proxies)
-        gobject.GObject.__init__ (self)
+        
         self._account = account
         self._thread = None
         self._authentication_retries = 0
         self._success = True
+        self._extra_widget_factory = extra_widget_factory
         
         LOGGER.debug ("Using proxies: %s" % proxies)
         
@@ -253,6 +259,9 @@ class GnomeURLopener (urllib.FancyURLopener, gobject.GObject):
                                          dialog_type=gtk.MESSAGE_WARNING)
             login_dialog.set_markup (_("<big><b>Login to %s rejected</b></big>") % self._account.get_host())
             login_dialog.format_secondary_markup (_("Please verify your credentials for <b>%s</b>") % self._account.get_host())
+            
+            self._install_extra_widget (login_dialog)
+            
             login_dialog.show_all()
             login_dialog.run()            
             login_dialog.destroy()
@@ -267,6 +276,9 @@ class GnomeURLopener (urllib.FancyURLopener, gobject.GObject):
             LOGGER.debug ("No credentials for %s in keyring. Asking for them..." %
                           self._account.get_host())
             login_dialog = AccountDialog(self._account)
+            
+            self._install_extra_widget (login_dialog)
+            
             login_dialog.show_all()
             login_dialog.run()            
             login_dialog.destroy()
@@ -281,7 +293,21 @@ class GnomeURLopener (urllib.FancyURLopener, gobject.GObject):
         
         return creds
     
-    def open_async (self, url, payload=None):
+    def _install_extra_widget (self, login_dialog):
+        """
+        Install the extra widget returned by the extra_widget_factory into
+        login_dialog, if applicable
+        """
+        if callable(self._extra_widget_factory):
+                widget = self._extra_widget_factory()
+                if widget != None:
+                    if isinstance (widget, gtk.Widget):
+                        login_dialog.vbox.pack_start (widget)
+                    else:
+                        # The factory returned a non-None non-widget object
+                        LOGGER.error ("%s returned a non-gtk.Widget instance: %s" % (self._extra_widget_factory, type(widget)))
+    
+    def open_async (self, url, payload=None, callback=None):
         """
         Open a URL asynchronously. When the request has been completed the
         C{"done"} signal of this class is emitted.
@@ -304,9 +330,9 @@ class GnomeURLopener (urllib.FancyURLopener, gobject.GObject):
             raise ConcurrentRequestsException()
     
         if payload != None :
-            async_args = (url, payload)
+            async_args = ((url, payload), callback)
         else :
-            async_args = (url, )
+            async_args = ((url, ), callback)
         
         self._thread = threading.Thread (target=self._do_open_async,
                                          args=async_args,
@@ -325,9 +351,10 @@ class GnomeURLopener (urllib.FancyURLopener, gobject.GObject):
         self._thread = None
         
         info = None
+        open_args, callback = args
         
         try:
-            info = self.open (*args)
+            info = self.open (*open_args)
         except AuthenticationAborted:
             LOGGER.debug ("Detected authentication abort")
             self._success = True # The user should not be warned
@@ -336,6 +363,7 @@ class GnomeURLopener (urllib.FancyURLopener, gobject.GObject):
             self._success = False            
             
         gtk.gdk.threads_enter()
-        self.emit ("done", info)
+        if callback != None and callable(callback):
+            callback(self, info)
         gtk.gdk.threads_leave()
 
